
Option Explicit On 

Imports Interop.QBFC5

Module modMaintenance

    Structure AlphaNumericRange
        Dim beginsAt As String
        Dim endsAt As String
        Public Sub New(ByVal b As String, ByVal e As String)
            beginsAt = b
            endsAt = e
        End Sub
    End Structure

    Sub maintProductsSeq()
        Dim sessionManager As QBSessionManager
        Dim msgSetRs As IMsgSetResponse

        Try
            sessionManager = New QBSessionManager
            Dim msgSetRq As IMsgSetRequest = sessionManager.CreateMsgSetRequest("US", 5, 0)
            sessionManager.OpenConnection("osc2QB", "osCommerce Integration Program")
            sessionManager.BeginSession("", ENOpenMode.omDontCare)

            msgSetRq.Attributes.OnError = ENRqOnError.roeContinue
            Dim messagesPresent As Boolean = False

            Dim ranges(9) As AlphaNumericRange
            ranges(0) = New AlphaNumericRange("0", "1")
            ranges(1) = New AlphaNumericRange("1", "1999")
            ranges(2) = New AlphaNumericRange("2", "2999")
            ranges(3) = New AlphaNumericRange("3", "3999")
            ranges(4) = New AlphaNumericRange("4", "4999")
            ranges(5) = New AlphaNumericRange("5", "5999")
            ranges(6) = New AlphaNumericRange("6", "6999")
            ranges(7) = New AlphaNumericRange("7", "7999")
            ranges(8) = New AlphaNumericRange("8", "8999")
            ranges(9) = New AlphaNumericRange("9", "9999")

            pgBar.Maximum = ranges.Length - 1
            pgBar.Value = 0
            Dim sql As String
            sql = "UPDATE products SET qb_list_id=null, qb_edit_seq=null "
            Debug.WriteLine(sql)
            dbConn.Execute(sql)

            Dim i As Integer
            For i = 0 To ranges.Length - 1
                Application.DoEvents()
                Debug.Write("=================================" & vbCrLf)
                Debug.Write(ranges(i).beginsAt & " to " & ranges(i).endsAt & vbCrLf)
                Debug.Write("=================================" & vbCrLf)
                msgSetRq.ClearRequests()

                DoItemInventoryQueryRq(ranges(i).beginsAt, ranges(i).endsAt, msgSetRq)
                msgSetRs = sessionManager.DoRequests(msgSetRq)
                processItemInventoryResponse(msgSetRs)
                Debug.Write(msgSetRq.ToXMLString)
                Debug.Write(msgSetRs.ToXMLString)
                lbStatus.Text = "Phase " & msgSetRs.ResponseList.Count & " of " & ranges.Length
                pgBar.Value = i
            Next
            pgBar.Value = 0
            lbStatus.Text = "Product Sequences Updated"
        Catch ex As Exception
            MessageBox.Show(ex.Message & vbCrLf & vbCrLf & ex.StackTrace, "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        Finally
            If Not sessionManager Is Nothing Then
                sessionManager.EndSession()
                sessionManager.CloseConnection()
            End If
        End Try
    End Sub

    Sub maintCustomerSeq()
        Dim sessionManager As QBSessionManager
        Dim msgSetRs As IMsgSetResponse

        Try
            sessionManager = New QBSessionManagerClass
            Dim msgSetRq As IMsgSetRequest = sessionManager.CreateMsgSetRequest("US", 5, 0)
            sessionManager.OpenConnection("osc2QB", "osCommerce Integration Program")
            sessionManager.BeginSession("", ENOpenMode.omDontCare)

            msgSetRq.Attributes.OnError = ENRqOnError.roeContinue
            Dim messagesPresent As Boolean = False

            Dim ranges(7) As AlphaNumericRange
            ranges(0) = New AlphaNumericRange("0", "9Z")
            ranges(1) = New AlphaNumericRange("A", "CZ")
            ranges(2) = New AlphaNumericRange("D", "GZ")
            ranges(3) = New AlphaNumericRange("H", "KZ")
            ranges(4) = New AlphaNumericRange("L", "OZ")
            ranges(5) = New AlphaNumericRange("P", "SZ")
            ranges(6) = New AlphaNumericRange("T", "WZ")
            ranges(7) = New AlphaNumericRange("X", "ZZ")

            pgBar.Maximum = ranges.Length - 1
            pgBar.Value = 0

            Dim sql As String
            sql = "UPDATE customers SET qb_list_id=null, qb_edit_seq=null "
            Debug.WriteLine(sql)
            dbConn.Execute(sql)

            Dim i As Integer
            For i = 0 To ranges.Length - 1
                Application.DoEvents()
                Debug.Write("=================================" & vbCrLf)
                Debug.Write(ranges(i).beginsAt & " to " & ranges(i).endsAt & vbCrLf)
                Debug.Write("=================================" & vbCrLf)
                msgSetRq.ClearRequests()
                DoCustomerQueryRq(ranges(i).beginsAt, ranges(i).endsAt, msgSetRq)
                msgSetRs = sessionManager.DoRequests(msgSetRq)
                processCustomerResponse(msgSetRs)
                Debug.Write(msgSetRs.ToXMLString)
                lbStatus.Text = "Phase " & msgSetRs.ResponseList.Count & " of " & ranges.Length
                pgBar.Value = i
            Next
            pgBar.Value = 0
            lbStatus.Text = "Customer Sequences Updated"
        Catch ex As Exception
            MessageBox.Show(ex.Message & vbCrLf & vbCrLf & ex.StackTrace, "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        Finally
            If Not sessionManager Is Nothing Then
                sessionManager.EndSession()
                sessionManager.CloseConnection()
            End If
        End Try

    End Sub

    Sub maintQBItemQuantity()
        Dim strSQL As String
        Dim dbCommand As New ADODB.Command
        Dim dbRs As New ADODB.Recordset
        Dim sessionManager As QBSessionManager
        Dim msgSetRs As IMsgSetResponse

        ' Query the database
        strSQL = "SELECT " & _
                    "p.products_id, p.products_quantity, p.qb_list_id, p.qb_edit_seq " & _
                "FROM `products` p " & _
                "WHERE qb_list_id is not null"
        dbCommand.ActiveConnection = dbConn

        dbRs.CursorType = ADODB.CursorTypeEnum.adOpenStatic
        lbStatus.Text = "Database cursor opened in static mode"
        dbRs.LockType = ADODB.LockTypeEnum.adLockPessimistic
        lbStatus.Text = "Pesimistic Lock set"

        dbCommand.CommandText = "SELECT " & _
                    "count(*) " & _
                "FROM `products` p " & _
                "WHERE qb_list_id is not null"
        dbRs = dbCommand.Execute

        dbRs.MoveFirst()
        Dim itemCount = dbRs.Fields(0).Value
        dbCommand.CommandText = strSQL
        dbRs = dbCommand.Execute
        pgBar.Maximum = itemCount
        Try
            sessionManager = New QBSessionManagerClass
            Dim msgSetRq As IMsgSetRequest = sessionManager.CreateMsgSetRequest("US", 5, 0)
            sessionManager.OpenConnection("osc2QB", "osCommerce Integration Program")
            sessionManager.BeginSession("", ENOpenMode.omDontCare)

            msgSetRq.Attributes.OnError = ENRqOnError.roeContinue
            Dim messagesPresent As Boolean = False
            Dim curInventoryItem As IItemInventoryRet
            Dim i = 0
            Dim oscInv As Integer
            Dim qbInv As Integer
            Dim invAdj As InventoryAdjustment
            While Not dbRs.EOF
                i = i + 1
                pgBar.Value = i
                lbStatus.Text = "Updating Product ID " & dbRs.Fields("products_id").Value
                Application.DoEvents()
                msgSetRq.ClearRequests()
                BuildItemInventoryQueryRq(dbRs.Fields("qb_list_id").Value, msgSetRq)

                Application.DoEvents()
                msgSetRs = sessionManager.DoRequests(msgSetRq)

                curInventoryItem = parseItemInventoryQueryRet(msgSetRs)
                oscInv = CInt(dbRs.Fields("products_quantity").Value)
                qbInv = CInt(curInventoryItem.QuantityOnHand.GetValue)
                Debug.WriteLine("QB:" & qbInv & ", OSC:" & oscInv)
                If Not qbInv = oscInv Then
                    invAdj = New InventoryAdjustment
                    invAdj.type = 0
                    invAdj.productListId = dbRs.Fields("qb_list_id").Value
                    invAdj.quantity = dbRs.Fields("products_quantity").Value
                    invAdj.memo = "Synchronization"
                    msgSetRq.ClearRequests()
                    BuildInventoryAdjustmentAddRq(invAdj, msgSetRq)
                    Application.DoEvents()
                    msgSetRs = sessionManager.DoRequests(msgSetRq)

                End If
                dbRs.MoveNext()
            End While
            lbStatus.Text = "OSC to QB Inventory Synchronization Complete"
            pgBar.Value = 0
            dbRs.Close()
            sessionManager.EndSession()
            sessionManager.CloseConnection()
        Catch ex As Exception
            MessageBox.Show(ex.Message & vbCrLf & vbCrLf & ex.StackTrace, "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        Finally
            If Not sessionManager Is Nothing Then
                sessionManager.EndSession()
                sessionManager.CloseConnection()
            End If
        End Try

    End Sub

    Sub maintQBItemValue()
        Dim strSQL As String
        Dim dbCommand As New ADODB.Command
        Dim dbRs As New ADODB.Recordset
        Dim sessionManager As QBSessionManager
        Dim msgSetRs As IMsgSetResponse

        ' Query the database
        strSQL = "SELECT " & _
                    "p.products_id, p.products_quantity, p.products_cost, p.qb_list_id, p.qb_edit_seq " & _
                "FROM `products` p " & _
                "WHERE qb_list_id is not null"
        dbCommand.ActiveConnection = dbConn

        dbRs.CursorType = ADODB.CursorTypeEnum.adOpenStatic
        lbStatus.Text = "Database cursor opened in static mode"
        dbRs.LockType = ADODB.LockTypeEnum.adLockPessimistic
        lbStatus.Text = "Pesimistic Lock set"

        dbCommand.CommandText = "SELECT " & _
                    "count(*) " & _
                "FROM `products` p " & _
                "WHERE qb_list_id is not null"
        dbRs = dbCommand.Execute

        dbRs.MoveFirst()
        Dim itemCount = dbRs.Fields(0).Value
        dbCommand.CommandText = strSQL
        dbRs = dbCommand.Execute
        pgBar.Maximum = itemCount
        Try
            sessionManager = New QBSessionManagerClass
            Dim msgSetRq As IMsgSetRequest = sessionManager.CreateMsgSetRequest("US", 5, 0)
            sessionManager.OpenConnection("osc2QB", "osCommerce Integration Program")
            sessionManager.BeginSession("", ENOpenMode.omDontCare)

            msgSetRq.Attributes.OnError = ENRqOnError.roeContinue
            Dim messagesPresent As Boolean = False
            Dim curInventoryItem As IItemInventoryRet
            Dim i = 0
            Dim oscInv As Integer
            Dim qbInv As Integer
            Dim invAdj As InventoryAdjustment
            While Not dbRs.EOF
                i = i + 1
                pgBar.Value = i
                lbStatus.Text = "Updating Product ID " & dbRs.Fields("products_id").Value
                Application.DoEvents()
                msgSetRq.ClearRequests()
                BuildItemInventoryQueryRq(dbRs.Fields("qb_list_id").Value, msgSetRq)

                Application.DoEvents()
                msgSetRs = sessionManager.DoRequests(msgSetRq)

                curInventoryItem = parseItemInventoryQueryRet(msgSetRs)
                oscInv = CInt(dbRs.Fields("products_quantity").Value)
                qbInv = CInt(curInventoryItem.QuantityOnHand.GetValue)
                Debug.WriteLine("QB:" & qbInv & ", OSC:" & oscInv)
                '==================================================
                If CInt(curInventoryItem.AverageCost.GetValue) = 0 Then
                    Debug.WriteLine("Product ID: " & dbRs.Fields("products_id").Value)
                    invAdj = New InventoryAdjustment
                    invAdj.type = 3
                    invAdj.productListId = dbRs.Fields("qb_list_id").Value
                    invAdj.quantity = dbRs.Fields("products_quantity").Value
                    invAdj.memo = "Synchronization"
                    invAdj.value = CInt(dbRs.Fields("products_quantity").Value) * CDbl(dbRs.Fields("products_cost").Value)
                    msgSetRq.ClearRequests()
                    BuildInventoryAdjustmentAddRq(invAdj, msgSetRq)
                    Application.DoEvents()
                    msgSetRs = sessionManager.DoRequests(msgSetRq)
                End If
                '==================================================
                dbRs.MoveNext()
            End While
            lbStatus.Text = "Value Adjustments Complete"
            pgBar.Value = 0
            dbRs.Close()
            sessionManager.EndSession()
            sessionManager.CloseConnection()
        Catch ex As Exception
            MessageBox.Show(ex.Message & vbCrLf & vbCrLf & ex.StackTrace, "Error", MessageBoxButtons.OK, MessageBoxIcon.Exclamation)
        Finally
            If Not sessionManager Is Nothing Then
                sessionManager.EndSession()
                sessionManager.CloseConnection()
            End If
        End Try

    End Sub

End Module
